from __future__ import absolute_import, division, print_function, unicode_literals

import inspect

#  All possible class names must be inserted into the globals collection.
#  If there is a better way of doing this, please suggest!
from amaascore.assets.asset import Asset
from amaascore.assets.bond import BondCorporate, BondGovernment, BondMortgage
from amaascore.assets.bond_future import BondFuture
from amaascore.assets.bond_future_option import BondFutureOption
from amaascore.assets.bond_option import BondOption
from amaascore.assets.cfd import ContractForDifference
from amaascore.assets.currency import Currency
from amaascore.assets.custom_asset import CustomAsset
from amaascore.assets.derivative import Derivative
from amaascore.assets.energy_future import EnergyFuture
from amaascore.assets.equity import Equity
from amaascore.assets.etf import ExchangeTradedFund
from amaascore.assets.foreign_exchange import ForeignExchange, NonDeliverableForward
from amaascore.assets.fund import Fund
from amaascore.assets.future import Future
from amaascore.assets.future_option import FutureOption
from amaascore.assets.fx_option import ForeignExchangeOption
from amaascore.assets.index import Index
from amaascore.assets.index_future import IndexFuture
from amaascore.assets.interest_rate_future import InterestRateFuture
from amaascore.assets.listed_cfd import ListedContractForDifference
from amaascore.assets.listed_derivative import ListedDerivative
from amaascore.assets.real_asset import RealAsset
from amaascore.assets.real_estate import RealEstate
from amaascore.assets.sukuk import Sukuk
from amaascore.assets.synthetic import Synthetic
from amaascore.assets.synthetic_from_book import SyntheticFromBook
from amaascore.assets.synthetic_multi_leg import SyntheticMultiLeg
from amaascore.assets.wine import Wine


def json_to_asset(json_asset):
    # Iterate through the asset children, converting the various JSON attributes into the relevant class type
    for (collection_name, clazz) in Asset.children().items():
        children = json_asset.pop(collection_name, {})
        collection = {}
        for (child_type, child_json) in children.items():
            # Handle the case where there are multiple children for a given type - e.g. links
            if isinstance(child_json, list):
                child = set()
                for child_json_in_list in child_json:
                    child.add(clazz(**child_json_in_list))
            else:
                child = clazz(**child_json)
            collection[child_type] = child
        json_asset[collection_name] = collection
    clazz = globals().get(json_asset.get('asset_type'))
    if not clazz:
        raise ValueError('Missing Asset Type: %s' % json_asset.get('asset_type'))
    args = inspect.getargspec(clazz.__init__)
    # Some fields are always added in, even though they're not explicitly part of the constructor
    clazz_args = args.args + clazz.amaas_model_attributes()
    # is not None is important so it includes zeros and False
    constructor_dict = {arg: json_asset.get(arg) for arg in clazz_args
                        if json_asset.get(arg) is not None and arg != 'self'}
    asset = clazz(**constructor_dict)
    return asset



from __future__ import absolute_import, division, print_function, unicode_literals

from decimal import Decimal


class OptionMixin(object):

    @property
    def option_style(self):
        if hasattr(self, '_option_style'):
            return self._option_style

    @option_style.setter
    def option_style(self, option_style):
        if option_style in ['American', 'Bermudan', 'European']:
            self._option_style= option_style
        else:
            raise ValueError("Invalid value for option_style: %s" % option_style)

    @property
    def option_type(self):
        if hasattr(self, '_option_type'):
            return self._option_type

    @option_type.setter
    def option_type(self, option_type):
            if option_type in ['Put', 'Call']:
                self._option_type = option_type
            else:
                raise ValueError("Invalid value for option_type: %s" % option_type)

    @property
    def strike(self):
        if hasattr(self, '_strike'):
            return self._strike

    @strike.setter
    def strike(self, strike):
        """ Force strike to be a Decimal. """
        if strike:
            self._strike = Decimal(strike)




from __future__ import absolute_import, division, print_function, unicode_literals

from amaascore.parties.party import Party


class SubFund(Party):

    def __init__(self, asset_manager_id, party_id, description='', base_currency=None, party_status='Active',
                 addresses=None, comments=None, emails=None, links=None, references=None,
                 *args, **kwargs):
        super(SubFund, self).__init__(asset_manager_id=asset_manager_id, party_id=party_id,
                                      base_currency=base_currency, description=description,
                                      party_status=party_status, addresses=addresses, comments=comments,
                                      emails=emails, links=links, references=references, *args, **kwargs)



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import date
from decimal import Decimal

from amaascore.assets.future import Future


class EquityFuture(Future):

    def __init__(self, asset_manager_id, asset_id, settlement_type, contract_size, point_value, tick_size,
                 quote_unit=None, asset_issuer_id=None, asset_status='Active', issue_date=date.min,
                 expiry_date=date.max, currency=None,
                 display_name='', description='', country_id=None, venue_id=None,
                 links=None, references=None, *args, **kwargs):
        """

        :param asset_manager_id: The asset manager who owns the data for this EquityFuture
        :param asset_id: The owning asset manager's identifier for this EquityFuture
        :param expiry_date: The date on which this EquityFuture will expire
        :param asset_issuer_id: The owning asset manager's identifier for the issuer of this future
        :param asset_status: The status of this EquityFuture
        :param issue_date: The date on which this EquityFuture was created
        :param display_name: A human-readable name for this EquityFuture
        :param description: A human-readable description of this EquityFuture
        :param country_id: The ISO 3166-1 alpha-3 code for the country that this EquityFuture was issued in
        :param venue_id: The ISO-10383 identifier for the venue on which this EquityFuture trades
        :param references: A dictionary of additional References for this EquityFuture
        :param args:
        :param kwargs:
        """
        super(EquityFuture, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                           settlement_type=settlement_type, contract_size=contract_size,
                                           point_value=point_value, tick_size=tick_size, quote_unit=quote_unit,
                                           asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                           display_name=display_name, currency=currency,
                                           description=description, country_id=country_id, venue_id=venue_id,
                                           links=links, references=references, issue_date=issue_date,
                                           expiry_date=expiry_date,
                                           *args, **kwargs)



# POTENTIALLY SUPPORT MULTIPLE ENVIRONMENTS
from __future__ import absolute_import, division, print_function, unicode_literals
LOCAL = False

envs = {'dev', 'staging'}

ENVIRONMENT = 'staging'

if LOCAL:
    ENDPOINTS = {
        'asset_managers': 'http://localhost:8000',
        'assets': 'http://localhost:8000',
        'books': 'http://localhost:8000',
        'corporate_actions': 'http://localhost:8000',
        'fundamentals': 'http://localhost:8000',
        'market_data': 'http://localhost:8000',
        'monitor': 'http://localhost:8000',
        'parties': 'http://localhost:8000',
        'transactions': 'http://localhost:8000'
    }
else:
    ENDPOINTS = {
        'asset_managers': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/assetmanager',
        'assets': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/asset',
        'books': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/book',
        'corporate_actions': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/corporateaction',
        'fundamentals': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/fundamental',
        'market_data': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/marketdata',
        'monitor': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/monitor',
        'parties': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/party',
        'transactions': 'https://iwe48ph25i.execute-api.ap-southeast-1.amazonaws.com/%s/transaction'
    }

COGNITO_CLIENT_ID = '55n70ns9u5stie272e1tl7v32v'  # This is not secret - it is just an identifier

# Do not change this
COGNITO_REGION = 'us-west-2'
COGNITO_POOL = 'us-west-2_wKa82vECF'



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import date, datetime
from dateutil.parser import parse
from decimal import Decimal
import pytz
import sys

from amaascore.core.amaas_model import AMaaSModel

# This extremely ugly hack is due to the whole Python 2 vs 3 debacle.
type_check = str if sys.version_info >= (3, 0, 0) else (str, unicode)


class FXRate(AMaaSModel):

    def __init__(self, asset_manager_id, asset_id, business_date, rate_timestamp, rate, rate_type, active=True,
                 *args, **kwargs):
        """

        :param asset_manager_id: The asset_manager_id who owns this price
        :param asset_id:  The asset_id for which we are providing a price
        :param business_date:  The business date for which this is the EOD price
        :param rate_timestamp: The specific point in time at which the rate was taken
        :param rate: The rate itself
        :param rate_type: An identifying "type" for the rate in case there are multiple rates for the same time period
        :param active: A boolean indicator of whether this rate is valid, or whether it was deactivated
        """
        self.asset_manager_id = asset_manager_id
        self.asset_id = asset_id
        self.business_date = business_date
        self.rate_timestamp = rate_timestamp
        self.rate_type = rate_type
        self.rate = rate
        self.active = active
        super(FXRate, self).__init__(*args, **kwargs)

    @property
    def rate(self):
        return self._rate

    @rate.setter
    def rate(self, value):
        """
        Force the rate to always be a decimal
        :param value:
        :return:
        """
        if value is not None:
            self._rate = Decimal(value)

    @property
    def business_date(self):
        return self._business_date

    @business_date.setter
    def business_date(self, business_date):
        """
        Force the business_date to always be a date
        :param business_date:
        :return:
        """
        if business_date is not None:
            if isinstance(business_date, type_check):
                self._business_date = parse(business_date).date()
            else:
                self._business_date= business_date

    @property
    def rate_timestamp(self):
        return self._rate_timestamp

    @rate_timestamp.setter
    def rate_timestamp(self, rate_timestamp):
        """
        Force the rate_timestamp to be a datetime
        :param rate_timestamp:
        :return:
        """
        if rate_timestamp is not None:
            if isinstance(rate_timestamp, (str, type_check)):
                rate_timestamp = parse(rate_timestamp).replace(tzinfo=pytz.utc)
            if type(rate_timestamp) == date:
                rate_timestamp = datetime.combine(rate_timestamp, datetime.min.time()).replace(tzinfo=pytz.utc)
            if not rate_timestamp.tzinfo:
                raise ValueError('Cannot set an FX rate timestamp without a timezone')
            self._rate_timestamp = rate_timestamp



from __future__ import absolute_import, division, print_function, unicode_literals

import logging

from amaascore.asset_managers.utils import json_to_asset_manager, json_to_relationship
from amaascore.config import ENVIRONMENT
from amaascore.core.interface import Interface


class AssetManagersInterface(Interface):
    """
    The interface to the Asset Managers service for reading Asset Manager information.
    """

    def __init__(self, environment=ENVIRONMENT, logger=None, endpoint=None):
        self.logger = logger or logging.getLogger(__name__)
        super(AssetManagersInterface, self).__init__(endpoint=endpoint, endpoint_type='asset_managers',
                                                     environment=environment)

    def new(self, asset_manager):
        self.logger.info('New Asset Manager: %s', asset_manager.asset_manager_id)
        url = '%s/asset-managers' % self.endpoint
        response = self.session.post(url, json=asset_manager.to_interface())
        if response.ok:
            self.logger.info('Successfully Created Asset Manager: %s', asset_manager.asset_manager_id)
            return json_to_asset_manager(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def retrieve(self, asset_manager_id):
        self.logger.info('Retrieve Asset Manager: %s', asset_manager_id)
        url = '%s/asset-managers/%s' % (self.endpoint, asset_manager_id)
        response = self.session.get(url)
        if response.ok:
            self.logger.info('Successfully Retrieved Asset Manager: %s', asset_manager_id)
            return json_to_asset_manager(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def deactivate(self, asset_manager_id):
        """
        Is is only possible to deactivate an asset manager if your client_id is also the client_id that was used
        to originally create the asset manager.

        :param asset_manager_id:
        :return:
        """
        self.logger.info('Deactivate Asset Manager: %s', asset_manager_id)
        url = '%s/asset-managers/%s' % (self.endpoint, asset_manager_id)
        response = self.session.delete(url)
        if response.ok:
            self.logger.info('Successfully deactivated Asset Manager: %s', asset_manager_id)
            return json_to_asset_manager(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def search(self, asset_manager_ids=None, client_ids=None):
        self.logger.info('Search for Asset Managers: %s', asset_manager_ids)
        search_params = {}
        # Potentially roll this into a loop through args rather than explicitly named - depends on additional validation
        if asset_manager_ids:
            search_params['asset_manager_ids'] = asset_manager_ids
        if client_ids:
            search_params['client_ids'] = client_ids
        url = self.endpoint + '/asset-managers'
        response = self.session.get(url, params=search_params)
        if response.ok:
            asset_managers = [json_to_asset_manager(json_asset_manager) for json_asset_manager in response.json()]
            self.logger.info('Returned %s Asset Managers.', len(asset_managers))
            return asset_managers
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def new_relationship(self, relationship):
        self.logger.info('New Asset Manager Relationship: %s and %s', relationship.asset_manager_id,
                         relationship.related_id)
        url = '%s/asset-manager-relationships/%s' % (self.endpoint, relationship.asset_manager_id)
        response = self.session.post(url, json=relationship.to_interface())
        if response.ok:
            self.logger.info('Successfully Created Asset Manager Relationship: %s', relationship.asset_manager_id)
            return json_to_relationship(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def amend_relationship(self, relationship):
        self.logger.info('Amend Asset Manager Relationship: %s and %s', relationship.asset_manager_id,
                         relationship.related_id)
        url = '%s/asset-manager-relationships/%s' % (self.endpoint, relationship.asset_manager_id)
        response = self.session.put(url, json=relationship.to_interface())
        if response.ok:
            self.logger.info('Successfully Amended Asset Manager Relationship: %s', relationship.asset_manager_id)
            return json_to_relationship(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def retrieve_relationships(self, asset_manager_id, related_id=None, include_inactive=False):
        self.logger.info('Retrieve Asset Manager Relationship: %s', asset_manager_id)
        url = '%s/asset-manager-relationships/%s' % (self.endpoint, asset_manager_id)
        params = {'include_inactive': include_inactive}
        if related_id:
            params['related_id'] = related_id
        response = self.session.get(url, params=params)
        if response.ok:
            self.logger.info('Successfully Amended Asset Manager Relationship: %s', asset_manager_id)
            return [json_to_relationship(json_relationship) for json_relationship in response.json()]
        else:
            self.logger.error(response.text)
            response.raise_for_status()



from __future__ import absolute_import, division, print_function, unicode_literals

WINE_CLASSIFICATIONS = {'1er Cru', '1er Cru Classé', '1er Grand Cru', '2ème Cru Classé', '2ème Vin',
                        '4ème Cru Classé', 'Cru Classé', 'Grand Cru', 'Non Classé'}
WINE_PACKING_TYPE = {'OCB06', 'OCB12', 'OWC06', 'OWC01', 'OWC02', 'OWC03', 'OWC06', 'OWC12'}



from __future__ import absolute_import, division, print_function, unicode_literals

TRANSACTION_TYPES = {'Allocation', 'Block', 'Cashflow', 'Coupon', 'Dividend', 'Exercise', 'Expiry', 'Payment',
                     'Journal', 'Maturity', 'Net', 'Novation', 'Split', 'Trade', 'Transfer'}
TRANSACTION_INVESTOR_ACTIONS = {'Subscription', 'Redemption'}
TRANSACTION_LIFECYCLE_ACTIONS = {'Acquire', 'Remove'}
TRANSACTION_ACTIONS = {'Buy', 'Sell', 'Short Sell', 'Deliver', 'Receive'} | TRANSACTION_LIFECYCLE_ACTIONS | \
                      TRANSACTION_INVESTOR_ACTIONS
TRANSACTION_CANCEL_STATUSES = {'Cancelled', 'Netted', 'Novated'}
TRANSACTION_STATUSES = {'New', 'Amended', 'Superseded'} | TRANSACTION_CANCEL_STATUSES



from __future__ import absolute_import, division, print_function, unicode_literals

import json
import logging

from amaascore.assets.utils import json_to_asset
from amaascore.config import ENVIRONMENT
from amaascore.core.interface import Interface
from amaascore.core.amaas_model import json_handler


class AssetsInterface(Interface):

    def __init__(self, environment=ENVIRONMENT, endpoint=None, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        super(AssetsInterface, self).__init__(endpoint=endpoint, endpoint_type='assets', environment=environment)

    def new(self, asset):
        self.logger.info('New Asset - Asset Manager: %s - Asset ID: %s', asset.asset_manager_id, asset.asset_id)
        url = '%s/assets/%s' % (self.endpoint, asset.asset_manager_id)
        response = self.session.post(url, json=asset.to_interface())
        if response.ok:
            self.logger.info('Successfully Created Asset - Asset Manager: %s - Asset ID: %s', asset.asset_manager_id,
                             asset.asset_id)
            asset = json_to_asset(response.json())
            return asset
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def amend(self, asset):
        self.logger.info('Amend Asset - Asset Manager: %s - Asset ID: %s', asset.asset_manager_id, asset.asset_id)
        url = '%s/assets/%s/%s' % (self.endpoint, asset.asset_manager_id, asset.asset_id)
        response = self.session.put(url, json=asset.to_interface())
        if response.ok:
            self.logger.info('Successfully Amended Asset - Asset Manager: %s - Asset ID: %s', asset.asset_manager_id,
                             asset.asset_id)
            asset = json_to_asset(response.json())
            return asset
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def partial(self, asset_manager_id, asset_id, updates):
        self.logger.info('Partial Amend Asset - Asset Manager: %s - Asset ID: %s', asset_manager_id,
                         asset_id)
        url = '%s/assets/%s/%s' % (self.endpoint, asset_manager_id, asset_id)
        # Setting handler ourselves so we can be sure Decimals work
        response = self.session.patch(url, data=json.dumps(updates, default=json_handler), headers=self.json_header)
        if response.ok:
            asset = json_to_asset(response.json())
            return asset
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def retrieve(self, asset_manager_id, asset_id):
        self.logger.info('Retrieve Asset - Asset Manager: %s - Asset ID: %s', asset_manager_id, asset_id)
        url = '%s/assets/%s/%s' % (self.endpoint, asset_manager_id, asset_id)
        response = self.session.get(url)
        if response.ok:
            self.logger.info('Successfully Retrieved Asset - Asset Manager: %s - Asset ID: %s', asset_manager_id,
                             asset_id)
            return json_to_asset(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def deactivate(self, asset_manager_id, asset_id):
        self.logger.info('Deactivate Asset - Asset Manager: %s - Asset ID: %s', asset_manager_id, asset_id)
        url = '%s/assets/%s/%s' % (self.endpoint, asset_manager_id, asset_id)
        json = {'asset_status': 'Inactive'}
        response = self.session.patch(url, json=json)
        if response.ok:
            self.logger.info('Successfully Deactivated Asset - Asset Manager: %s - Asset ID: %s', asset_manager_id,
                             asset_id)
            return json_to_asset(response.json())
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def search(self, asset_manager_ids=None, asset_ids=None):
        self.logger.info('Search for Assets - Asset Manager(s): %s', asset_manager_ids)
        search_params = {}
        # Potentially roll this into a loop through args rather than explicitly named - depends on additional validation
        if asset_manager_ids:
            search_params['asset_manager_ids'] = asset_manager_ids
        if asset_ids:
            search_params['asset_ids'] = asset_ids
        url = self.endpoint + '/assets'
        response = self.session.get(url, params=search_params)
        if response.ok:
            assets = [json_to_asset(json_asset) for json_asset in response.json()]
            self.logger.info('Returned %s Assets.', len(assets))
            return assets
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def assets_by_asset_manager(self, asset_manager_id):
        url = '%s/assets/%s' % (self.endpoint, asset_manager_id)
        response = self.session.get(url)
        if response.ok:
            assets = [json_to_asset(json_asset) for json_asset in response.json()]
            self.logger.info('Returned %s Assets.', len(assets))
            return assets
        else:
            self.logger.error(response.text)
            response.raise_for_status()



from __future__ import absolute_import, division, print_function, unicode_literals

import json
import logging

from amaascore.config import ENVIRONMENT
from amaascore.core.amaas_model import json_handler
from amaascore.core.interface import Interface
from amaascore.market_data.utils import json_to_eod_price, json_to_fx_rate


class MarketDataInterface(Interface):

    def __init__(self, environment=ENVIRONMENT, logger=None, endpoint=None):
        self.logger = logger or logging.getLogger(__name__)
        super(MarketDataInterface, self).__init__(endpoint=endpoint, endpoint_type='market_data',
                                                  environment=environment)

    def persist_eod_prices(self, asset_manager_id, business_date, eod_prices, update_existing_prices=True):
        """

        :param asset_manager_id:
        :param business_date: The business date for which these are rates.  Not really needed, could be derived...
        :param eod_prices:
        :param update_existing_prices:
        :return:
        """
        self.logger.info('Persist EOD Prices - Asset Manager: %s - Business Date: %s', asset_manager_id, business_date)
        url = '%s/eod-prices/%s/%s' % (self.endpoint, asset_manager_id, business_date.isoformat())
        params = {'update_existing_prices': update_existing_prices}
        eod_prices_json = [eod_price.to_interface() for eod_price in eod_prices]
        response = self.session.post(url, params=params, json=eod_prices_json)
        if response.ok:
            eod_prices = [json_to_eod_price(eod_price) for eod_price in response.json()]
            return eod_prices
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def retrieve_eod_prices(self, asset_manager_id, business_date, asset_ids=None):
        url = '%s/eod-prices/%s/%s' % (self.endpoint, asset_manager_id, business_date.isoformat())
        params = {'asset_ids': ','.join(asset_ids)} if asset_ids else {}
        response = self.session.get(url=url, params=params)
        if response.ok:
            eod_prices = [json_to_eod_price(eod_price) for eod_price in response.json()]
            self.logger.info('Returned %s EOD Prices.', len(eod_prices))
            return eod_prices
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def roll_prices(self, asset_manager_id, previous_date, asset_ids, update_existing_prices=False):
        url = '%s/roll-prices/%s' % (self.endpoint, asset_manager_id)
        params = {'update_existing_prices': update_existing_prices}
        json_body = json.loads(json.dumps({'previous_date': previous_date,
                                           'asset_ids': ','.join(asset_ids)}, default=json_handler))
        response = self.session.post(url=url, params=params, json=json_body)
        if response.ok:
            prices = [json_to_eod_price(price) for price in response.json()]
            self.logger.info('Rolled %s Prices.', len(prices))
            return prices
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def persist_fx_rates(self, asset_manager_id, business_date, fx_rates, update_existing_rates=True):
        """

        :param asset_manager_id:
        :param business_date: The business date for which these are rates.  Not really needed, could be derived...
        :param fx_rates:
        :param update_existing_rates:
        :return:
        """
        url = '%s/fx-rates/%s/%s' % (self.endpoint, asset_manager_id, business_date.isoformat())
        params = {'update_existing_rates': update_existing_rates}
        fx_rates_json = [fx_rate.to_interface() for fx_rate in fx_rates]
        response = self.session.post(url, params=params, json=fx_rates_json)
        if response.ok:
            fx_rates = [json_to_fx_rate(fx_rate) for fx_rate in response.json()]
            return fx_rates
        else:
            self.logger.error(response.text)
            response.raise_for_status()

    def retrieve_fx_rates(self, asset_manager_id, business_date, asset_ids=None):
        url = '%s/fx-rates/%s/%s' % (self.endpoint, asset_manager_id, business_date.isoformat())
        params = {'asset_ids': ','.join(asset_ids)} if asset_ids else {}
        response = self.session.get(url=url, params=params)
        if response.ok:
            fx_rates = [json_to_fx_rate(fx_rate) for fx_rate in response.json()]
            self.logger.info('Returned %s FX Rates.', len(fx_rates))
            return fx_rates
        else:
            self.logger.error(response.text)
            response.raise_for_status()



from __future__ import absolute_import, division, print_function, unicode_literals

from amaasutils.random_utils import random_string, random_date
import random

from amaascore.assets.enums import WINE_CLASSIFICATIONS, WINE_PACKING_TYPE
from amaascore.assets.wine import Wine
from amaascore.core.comment import Comment
from amaascore.tools.generate_asset import generate_common


def generate_wine(asset_manager_id=None, asset_id=None):
    props = generate_common(asset_manager_id=asset_manager_id, asset_id=asset_id)
    wine = Wine(year=random_date(start_year=1950, end_year=2016),
                producer=random_string(5),
                region=random.choice(['Bordeaux', 'Burgundy', 'Tuscany', 'Napa Valley']),
                appellation=random.choice([None]*3 + ['Côtes du Rhône', 'Graves', 'Saint-Émilion']),
                classification=random.choice(list(WINE_CLASSIFICATIONS)),
                color=random.choice(['Red', 'White']),
                bottle_size=random.choice(['0.75L']*3 + ['1.5L']),
                bottle_in_cellar=random.choice([True]*3 + [False]),
                bottle_location=random_string(20),
                storage_cost=None,
                rating_type='Parker',
                rating_value=random.randint(93, 100),
                packing_type=random.choice(list(WINE_PACKING_TYPE)),
                to_drink_start=random_date(start_year=2000),
                to_drink_end=random_date(end_year=2050),
                comments = {'DrinkingNotes': Comment(comment_value=random_string(100))},
                **props)
    return wine



from __future__ import absolute_import, division, print_function, unicode_literals

import re

from amaascore.error_messages import ERROR_LOOKUP
from amaascore.core.amaas_model import AMaaSModel


class Address(AMaaSModel):

    def __init__(self, line_one, city, country_id, address_primary, line_two=None, region=None, postal_code=None,
                 active=True, version=1, *args, **kwargs):
        self.address_primary = address_primary
        self.line_one = line_one
        self.line_two = line_two
        self.city = city
        self.region = region
        self.postal_code = postal_code
        self.country_id = country_id
        self.active = active
        self.version = version
        super(Address, self).__init__(*args, **kwargs)

    @property
    def country_id(self):
        if hasattr(self, '_country_id'):
            return self._country_id

    @country_id.setter
    def country_id(self, country_id):
        if country_id:
            if len(country_id) != 3:
                raise ValueError(ERROR_LOOKUP.get('country_id_invalid') % country_id)
            self._country_id = country_id


class Email(AMaaSModel):

    def __init__(self, email, email_primary, active=True, version=1, *args, **kwargs):
        self.email_primary = email_primary
        self.email = email
        self.active = active
        self.version = version
        super(Email, self).__init__(*args, **kwargs)

    @property
    def email(self):
        if hasattr(self, '_email'):
            return self._email

    @email.setter
    def email(self, email):
        # Validate email addresses
        if not re.match('[^@]+@[^@]+\.[^@]+', email):
            raise ValueError(ERROR_LOOKUP.get('email_address_invalid') % email)
        self._email = email


class Link(AMaaSModel):

    def __init__(self, linked_party_id, active=True, version=1, *args, **kwargs):
        self.linked_party_id = linked_party_id
        self.active = active
        self.version = version
        super(Link, self).__init__(*args, **kwargs)



from __future__ import absolute_import, division, print_function, unicode_literals

from amaascore.core.amaas_model import AMaaSModel


class Link(AMaaSModel):

    def __init__(self, linked_asset_id, active=True, version=1, *args, **kwargs):
        self.linked_asset_id = linked_asset_id
        self.active = active
        self.version = version
        super(Link, self).__init__(*args, **kwargs)



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import date

from amaascore.assets.synthetic import Synthetic


class SyntheticFromBook(Synthetic):
    """ A synthetic asset whose value is based on the value of the assets in a referenced book """

    def __init__(self, asset_id, asset_manager_id, book_id=None, asset_issuer_id=None, asset_status='Active',
                 country_id=None, currency=None, display_name='', description='', fungible=True, issue_date=date.min,
                 maturity_date=date.max, comments=None, links=None, references=None, *args, **kwargs):
        self.book_id = book_id
        super(SyntheticFromBook, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                                fungible=fungible, asset_issuer_id=asset_issuer_id,
                                                asset_status=asset_status, currency=currency,
                                                issue_date=issue_date, maturity_date=maturity_date,
                                                country_id=country_id, roll_price=False,
                                                display_name=display_name, description=description,
                                                comments=comments, links=links, references=references,
                                                *args, **kwargs)



from __future__ import absolute_import, division, print_function, unicode_literals

from amaascore.monitor.item import Item


def json_to_item(json_item):
    item = Item(**json_item)
    return item



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import date
from decimal import Decimal

from amaascore.assets.derivative import Derivative
from amaascore.assets.option_mixin import OptionMixin


class ForeignExchangeOption(Derivative, OptionMixin):
    """
    An over the counter Option with an underlying FX pair.
    """

    def __init__(self, asset_manager_id, asset_id, option_type, strike, premium, underlying_asset_id, option_style,
                 issue_date=date.min, asset_status='Active', asset_issuer_id=None,
                 display_name='', description='', links=None, references=None,
                 *args, **kwargs):
        self.option_type = option_type
        self.strike = strike
        self.premium = premium
        self.underlying_asset_id = underlying_asset_id
        self.option_style = option_style
        super(ForeignExchangeOption, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                                    fungible=False, asset_issuer_id=asset_issuer_id,
                                                    issue_date=issue_date, asset_status=asset_status,
                                                    display_name=display_name, description=description,
                                                    links=links, references=references, *args, **kwargs)



from __future__ import absolute_import, division, print_function, unicode_literals

from datetime import date
from dateutil.parser import parse
import sys

from amaascore.assets.real_asset import RealAsset

# This extremely ugly hack is due to the whole Python 2 vs 3 debacle.
type_check = str if sys.version_info >= (3, 0, 0) else (str, unicode)


class Wine(RealAsset):

    def __init__(self, asset_manager_id, asset_id, year=None, producer=None,
                 region=None, appellation=None, classification=None, color=None, bottle_size=None,
                 bottle_in_cellar=None, bottle_location=None, storage_cost=None, rating_type=None,
                 rating_value=None, packing_type=None, to_drink_start=None, to_drink_end=None,
                 asset_issuer_id=None, asset_status='Active', display_name='', description='',
                 country_id=None, venue_id=None, currency=None, issue_date=date.min,
                 comments=None, links=None, references=None, *args, **kwargs):
        self.year = year
        self.producer = producer
        self.region = region
        self.appellation = appellation
        self.classification = classification
        self.color = color
        self.bottle_size = bottle_size
        self.bottle_in_cellar = bottle_in_cellar
        self.bottle_location = bottle_location
        self.storage_cost = storage_cost
        self.rating_type = rating_type
        self.rating_value = rating_value
        self.packing_type = packing_type
        self.to_drink_start = to_drink_start
        self.to_drink_end = to_drink_end
        super(Wine, self).__init__(asset_manager_id=asset_manager_id, asset_id=asset_id,
                                   asset_issuer_id=asset_issuer_id, asset_status=asset_status,
                                   display_name=display_name,
                                   description=description, country_id=country_id, venue_id=venue_id,
                                   issue_date=issue_date, currency=currency,
                                   comments=comments, links=links, references=references,
                                   *args, **kwargs)

    @property
    def year(self):
        if hasattr(self, '_year'):
            return self._year

    @year.setter
    def year(self, year):
        if isinstance(year, type_check):
            year = int(year)
        self._year = year

    @property
    def to_drink_start(self):
        if hasattr(self, '_to_drink_start'):
            return self._to_drink_start

    @to_drink_start.setter
    def to_drink_start(self, to_drink_start):
        if isinstance(to_drink_start, (type_check)):
            to_drink_start = parse(to_drink_start).date()
        self._to_drink_start = to_drink_start

    @property
    def to_drink_end(self):
        if hasattr(self, '_to_drink_end'):
            return self._to_drink_end

    @to_drink_end.setter
    def to_drink_end(self, to_drink_end):
        if isinstance(to_drink_end, (type_check)):
            to_drink_end = parse(to_drink_end).date()
        self._to_drink_end = to_drink_end

